## Copyright 2023 Jakob Harden, Graz University of Technology (Graz, Austria)
## This script is a part of and was written in the course of the PhD study of the author.
## License: MIT, see also section "License" in README.md
##
## Create reference to data structure (atomic reference element, ARE)
##
## Usage: [r_ds] = struct_objref(p_t, p_i, p_r, p_d)
##
## Note: p_r is a cell-array of strings. Each array items points to another level of
##        the structure hierarchy. Structure levels must be arranged in descending order
##        and the root of the hierarchy must not be referenced.
##        This definition allows to access any structure element by subsequent calls
##        to the "getfield(...)" command.
##        Example: reference to data structure element "test" in "root.lvl1.lvl2.test"
##                 results in: p_r = {"lvl1", "lvl2", "test"}
##
## p_t  ... tag, <str>
## p_i  ... id's of related data object(s), <uint> or [<uint>]
## p_r  ... structure path to related data object, {<str>}
## p_d  ... description, <str>
## r_ds ... return: data object, <ARE/struct>
##   .obj ... object type, "ARE"
##   .ver ... version numbers [main_ver, sub_ver], [<uint>]
##   .t ... tag, <str>
##   .i ... id of related data object, <uint>
##   .r ... reference target, {<str>}
##   .d ... description, <str>
##
## see also: struct_objdata, struct_objattr
##
function [r_ds] = struct_objref(p_t, p_i, p_r, p_d)
  
  ## check arguments
  if (nargin < 1)
    help struct_objref;
    error('Less arguments given!');
  endif
  if isempty(p_t)
    help struct_objref;
    error('Reference tag is empty!');
  endif
  if (nargin < 2)
    p_i = [];
  endif
  if (nargin < 3)
    p_r = '';
  endif
  if (nargin < 4)
    p_d = '';
    warning('Description of atomic reference element is empty!');
  endif
  
  ## check reference target value type
  if not(iscell(p_r))
    p_r = {p_r};
  endif
  
  ## check reference target index type
  if not(isempty(p_i))
    if not(isnumeric(p_i))
      help struct_objref;
      error("ID's to related data objects must be numeric values! (<uint> or [<uint>])");
    endif
  endif
  
  # create structure
  r_ds.obj = 'ARE';
  r_ds.ver = uint16([1, 0]);
  r_ds.t = p_t;
  r_ds.i = p_i;
  r_ds.r = p_r;
  r_ds.d = p_d;
  
endfunction
