## Copyright 2023 Jakob Harden, Graz University of Technology (Graz, Austria)
## This script is a part of and was written in the course of the PhD study of the author.
## License: MIT, see also section "License" in README.md
##
## Compile data structure from rawdata
##
function createds()
  
  ## parse files from rawdata directory
  d1 = csvread('./rawdata/raw01.csv');
  d2 = hlp_parse_par('./rawdata/par01.txt');
  
  ## create data structure
  ## substructure: data
  ds.data.xdata = struct_objdata('xdata', 'double_arr', d1(:, 1), d2.xunit.v, 'sample time [t]');
  ds.data.ydata = struct_objdata('ydata', 'double_mat', d1(:, 2:3), d2.yunit.v, 'sample magnitudes [s1, s2]');
  ds.data.creatorref = struct_objref('creatorref', 1, {'meta', 'creator'}, 'reference to raw data creator');
  ds.data.dateref = struct_objref('dateref', 1, {'meta', 'date'}, 'reference to creation date');
  ds.data.geoloclatref = struct_objref('geoloclatref', 1, {'meta', 'geoloclatitude'}, 'reference to geolocation latitude');
  ds.data.geoloclongref = struct_objref('geoloclongref', 1, {'meta', 'geoloclongitude'}, 'reference to geolocation longitude');
  ## substructure: metadata
  ds.meta = d2;
  
  ## print information on screen
  printf('Created metadata substructure\n');
  printf('Created data substructure\n');
  printf('Added reference to creator: %s\n', struct_objref_resolve(ds, ds.data.creatorref).v);
  printf('Added reference to date: %s\n', struct_objref_resolve(ds, ds.data.dateref).v);
  printf('Added reference to geolocation latitude: %.6f\n', struct_objref_resolve(ds, ds.data.geoloclatref).v);
  printf('Added reference to geolocation longitude: %.6f\n', struct_objref_resolve(ds, ds.data.geoloclongref).v);
  
  ## save binary file to output directory
  dp = './structure';
  if not(exist(dp, 'dir') == 7)
    mkdir(dp);
  endif
  save('-binary', fullfile(dp, 'ds.oct'), 'ds');

endfunction

function [r_ds] = hlp_parse_par(p_fp)
  ##
  ## helper function: parse parameter file
  ##
  ## p_fp ... parameter file path, <str>
  ## r_ds ... return: parameter data structure, <struct>
  ##
  
  ## init return value
  r_ds = [];
  
  ## open parameter file
  fid = fopen(p_fp, 'r');
  
  ## iterate over all lines
  while (feof(fid) != 1)
    ## read line of file
    ln = strtrim(fgetl(fid));
    ## split line
    [a1, a2] = strsplit(ln, {';'});
    ## trim portions
    v1 = strtrim(a1{1}); # data type
    v2 = strtrim(a1{2}); # parameter name, tag
    v3 = strtrim(a1{3}); # parameter value
    ## switch data types
    switch (v1)
      case 'double'
        val = str2num(v3);
        obj = struct_objdata(v2, v1, val, [], sprintf('parameter: %s', v2));
        r_ds = setfield(r_ds, v2, obj);
      case 'string'
        val = v3;
        obj = struct_objdata(v2, v1, val, [], sprintf('parameter: %s', v2));
        r_ds = setfield(r_ds, v2, obj);
      case 'uint'
        val = abs(fix(str2num(v3)));
        obj = struct_objdata(v2, v1, val, [], sprintf('parameter: %s', v2));
        r_ds = setfield(r_ds, v2, obj);
    endswitch
  endwhile
  
  ## close parameter file
  fclose(fid);
  
endfunction
